local Isbn = require "isbn"
require "isbn-test"
local testcase = TestCase("Test 10 digit ISBNs")

local GOOD_ISBN          = "0596527241"
local GOOD_ISBN_STRING   = "0-596-52724-1"
local GOOD_EAN           = "9780596527242"
local GOOD_EAN_STRING    = "978-0-596-52724-2"
local GROUP_NAME         = "English speaking area"
local PREFIX             = "978"
local GROUP_CODE         = "0"
local PUBLISHER          = "596"

local BAD_CHECKSUM_ISBN  = "0596527244"
local X_BAD_CHECKSUM_ISBN = "059652724X"
local MISSING_CHECKSUM_ISBN = "059652724"

function testcase:setup ()
    local err
    self.isbn, err = Isbn:new(GOOD_ISBN)
    assert(not err)
end

function testcase:test_construct_object ()
    assert_table(self.isbn)
    assert_isa(self.isbn, Isbn)
    is(GOOD_ISBN, self.isbn:isbn(), "isbn() method")
    is("", self.isbn:prefix(), "has right prefix")
    is(PUBLISHER, self.isbn:publisher_code(), "has right publisher")
    is(GROUP_CODE, self.isbn:group_code(), "has right country code")
    is(GROUP_NAME, self.isbn:group_name(), "has right country name")
    is(GOOD_ISBN_STRING, tostring(self.isbn), "stringifies correctly")
end

function testcase:test_clone ()
    local clone = self.isbn:as_isbn10()
    assert_isa(clone, Isbn)
    is(GOOD_ISBN, clone:isbn(), "isbn() method")
    is(PUBLISHER, clone:publisher_code(), "has right publisher")
    is(GROUP_CODE, clone:group_code(), "has right country code")
    is(GROUP_NAME, clone:group_name(), "has right country name")
    is(GOOD_ISBN_STRING, tostring(clone), "stringifies correctly")
end

function testcase:test_isbn13 ()
    local clone = self.isbn:as_isbn13()
    assert_isa(clone, Isbn)
    is(GOOD_EAN, clone:isbn(), "isbn() method")
    is(PREFIX, clone:prefix(), "has right prefix")
    is(PUBLISHER, clone:publisher_code(), "has right publisher")
    is(GROUP_CODE, clone:group_code(), "has right country code")
    is(GROUP_NAME, clone:group_name(), "has right country name")
    is(GOOD_EAN_STRING, tostring(clone), "stringifies correctly")
end

function testcase:test_completely_wrong ()
    local isbn, err = Isbn:new("abcdefghij")
    assert_nil(isbn)
    is("wrong number of digits", err)
    local isbn, err = Isbn:new("156592")
    assert_nil(isbn)
    is("wrong number of digits", err)
end

function testcase:test_bad_checksum ()
    local isbn, err = Isbn:new(BAD_CHECKSUM_ISBN)
    assert_nil(isbn)
    is("wrong checksum digit", err)
    isbn, err = Isbn:new(BAD_CHECKSUM_ISBN, "fix-checksum")
    assert_isa(isbn, Isbn)
    is(GOOD_ISBN, isbn:isbn(), "isbn() method")
    is(GOOD_ISBN_STRING, tostring(isbn), "stringifies correctly")
end

function testcase:test_bad_checksum_x ()
    local isbn, err = Isbn:new(X_BAD_CHECKSUM_ISBN)
    assert_nil(isbn)
    is("wrong checksum digit", err)
    isbn, err = Isbn:new(X_BAD_CHECKSUM_ISBN, "fix-checksum")
    assert_isa(isbn, Isbn)
    is(GOOD_ISBN, isbn:isbn(), "isbn() method")
    is(GOOD_ISBN_STRING, tostring(isbn), "stringifies correctly")
end

function testcase:test_missing_checksums ()
    local isbn, err = Isbn:new(MISSING_CHECKSUM_ISBN)
    assert_nil(isbn)
    is("wrong number of digits", err)
    isbn, err = Isbn:new(MISSING_CHECKSUM_ISBN, "fix-checksum")
    assert_isa(isbn, Isbn)
    is(GOOD_ISBN, isbn:isbn(), "isbn() method")
    is(GOOD_ISBN_STRING, tostring(isbn), "stringifies correctly")
end

function testcase:test_bad_group_code ()
    local isbn, err = Isbn:new("9990222576")
    assert_nil(isbn)
    is("invalid group code", err)
end

function testcase:test_bad_publisher_code ()
    local isbn, err = Isbn:new("9165022222")
    assert_nil(isbn)
    is("invalid publisher code", err)
end

function testcase:test_strict_and_lenient ()
    -- These extra characters are normally ignored, but not in strict mode.
    local GOOD_WITH_BAD_CHARS = " -0596%527  (ignored) 241\t\n\r"
    local isbn, err = Isbn:new(GOOD_WITH_BAD_CHARS)
    is(GOOD_ISBN, isbn:isbn())
    isbn, err = Isbn:new(GOOD_WITH_BAD_CHARS, "strict")
    assert_nil(isbn)
    is("character not allowed", err)

    -- These extra characters are always allowed, even in strict mode.
    local GOOD_WITH_SUPERFLUOUS_CHARS = " -0596 527- - 241\t\n\r"
    local isbn, err = Isbn:new(GOOD_WITH_SUPERFLUOUS_CHARS)
    is(GOOD_ISBN, isbn:isbn())
    isbn, err = Isbn:new(GOOD_WITH_SUPERFLUOUS_CHARS, "strict")
    is(GOOD_ISBN, isbn:isbn())
end

function testcase:convert_to_EAN ()
    local isbn = Isbn:new(GOOD_ISBN)
    is(GOOD_EAN, tostring(isbn:as_isbn13()), "converted to EAN")
end

function testcase:test_good_isbns ()
    local file = assert(io.open("test/data/isbns.txt", "rb"))

    for line in file:lines() do
        local isbn, err = Isbn:new(line)
        assert_table(isbn)
        assert_nil(err)

        isbn, err = Isbn:new("978" .. line, "fix-checksum")
        assert_table(isbn)
        assert_nil(err)
    end

    file:close()
end

function testcase:test_bad_isbns ()
    local file = assert(io.open("test/data/bad-isbns.txt", "rb"))

    for line in file:lines() do
        local isbn, err = Isbn:new(line)
        assert_nil(isbn, "value returned for bad ISBN-10: " .. line)
        assert_string(err, "no string for error message for ISBN-10: " .. line)
    end

    file:close()
end

lunit.run()
-- vi:ts=4 sw=4 expandtab
