local Isbn = require "isbn"
require "isbn-test"
local testcase = TestCase("Test 13 digit ISBNs")

local GOOD_ISBN          = "9780596527242"
local GOOD_ISBN_STRING   = "978-0-596-52724-2"
local GOOD_ISBN10        = "0596527241"
local PREFIX             = "978"
local GROUP_NAME         = "English speaking area"
local GROUP_CODE         = "0"
local PUBLISHER          = "596"
local ARTICLE            = "52724"
local CHECKSUM           = "2"

local BAD_CHECKSUM_ISBN  = "9780596527244"
local X_BAD_CHECKSUM_ISBN = "978059652724X"
local MISSING_CHECKSUM_ISBN = "978059652724"

function testcase:setup ()
    local err
    self.isbn, err = Isbn:new(GOOD_ISBN)
    assert(not err)
end

function testcase:test_construct_object ()
    assert_table(self.isbn)
    assert_isa(self.isbn, Isbn)
    is(GOOD_ISBN, self.isbn:isbn(), "isbn() method")
    is(PREFIX, self.isbn:prefix(), "has right prefix")
    is(GROUP_CODE, self.isbn:group_code(), "has right group code")
    is(GROUP_NAME, self.isbn:group_name(), "has right group name")
    is(PUBLISHER, self.isbn:publisher_code(), "has right publisher")
    is(ARTICLE, self.isbn:article_code(), "has right article")
    is(CHECKSUM, self.isbn:checksum(), "has right checksum")
    is(GOOD_ISBN_STRING, tostring(self.isbn), "stringify metamethod")
end

-- can I clone it?
function testcase:test_clone ()
    local clone = self.isbn:as_isbn13()
    assert_isa(clone, Isbn)
    is(GOOD_ISBN, clone:isbn(), "isbn() method")
    is(PREFIX, clone:prefix(), "has right prefix")
    is(GROUP_CODE, clone:group_code(), "has right group code")
    is(GROUP_NAME, clone:group_name(), "has right group name")
    is(PUBLISHER, clone:publisher_code(), "has right publisher")
    is(ARTICLE, clone:article_code(), "has right article")
    is(CHECKSUM, clone:checksum(), "has right checksum")
    is(GOOD_ISBN_STRING, tostring(clone), "stringifies correctly")
end

-- can I make it ISBN10?
function testcase:test_isbn10 ()
    local clone = self.isbn:as_isbn10()
    assert_isa(clone, Isbn)
    is(GOOD_ISBN10, clone:isbn(), "isbn() method")
    is("", clone:prefix(), "has right prefix")
    is(PUBLISHER, clone:publisher_code(), "has right publisher")
    is(GROUP_CODE, clone:group_code(), "has right country code")
    is(GROUP_NAME, clone:group_name(), "has right country name")
end

function testcase:test_bad_prefix ()
    local isbn, err = Isbn:new("1230439994149")
    assert_nil(isbn)
    is("invalid prefix", err)
end

function testcase:test_completely_wrong ()
    local isbn, err = Isbn:new("978abcdefghij")
    assert_nil(isbn)
    is("wrong number of digits", err)
    local isbn, err = Isbn:new("978156592")
    assert_nil(isbn)
    is("wrong number of digits", err)
end

function testcase:test_x_in_middle ()
    local isbn, err = Isbn:new("97805X6527242")
    assert_nil(isbn)
    is("digit 'X' not allowed in middle", err)
end

function testcase:test_bad_checksum ()
    local isbn, err = Isbn:new(BAD_CHECKSUM_ISBN)
    assert_nil(isbn)
    is("wrong checksum digit", err)
    isbn, err = Isbn:new(BAD_CHECKSUM_ISBN, "fix-checksum")
    assert_isa(isbn, Isbn)
    is(GOOD_ISBN, isbn:isbn(), "isbn() method")
    is(GOOD_ISBN_STRING, tostring(isbn), "stringifies correctly")
end

function testcase:test_bad_checksum_x ()
    -- The check digit 'X' is never correct on an ISBN-13, but it should
    -- be treated as a wrong checksum, and therefore can be corrected if
    -- you ask nicely.
    local isbn, err = Isbn:new(X_BAD_CHECKSUM_ISBN)
    assert_nil(isbn)
    is("wrong checksum digit", err)
    isbn, err = Isbn:new(X_BAD_CHECKSUM_ISBN, "fix-checksum")
    assert_isa(isbn, Isbn)
    is(GOOD_ISBN, isbn:isbn(), "isbn() method")
    is(GOOD_ISBN_STRING, tostring(isbn), "stringifies correctly")
end

function testcase:test_missing_checksums ()
    local isbn, err = Isbn:new(MISSING_CHECKSUM_ISBN)
    assert_nil(isbn)
    is("wrong number of digits", err)
    isbn, err = Isbn:new(MISSING_CHECKSUM_ISBN, "fix-checksum")
    assert_isa(isbn, Isbn)
    is(GOOD_ISBN, isbn:isbn(), "isbn() method")
    is(GOOD_ISBN_STRING, tostring(isbn), "stringifies correctly")
end

function testcase:test_bad_group_code ()
    local isbn, err = Isbn:new("978-9990222576")
    assert_nil(isbn)
    is("invalid group code", err)
end

function testcase:test_bad_publisher_code ()
    local isbn, err = Isbn:new("978-9165022222")
    assert_nil(isbn)
    is("invalid publisher code", err)
end

function testcase:test_strict_and_lenient ()
    -- These extra characters are normally ignored, but not in strict mode.
    local GOOD_WITH_BAD_CHARS = " 978-0596%527  (ignored) 242\t\n\r"
    local isbn, err = Isbn:new(GOOD_WITH_BAD_CHARS)
    is(GOOD_ISBN, isbn:isbn())
    isbn, err = Isbn:new(GOOD_WITH_BAD_CHARS, "strict")
    assert_nil(isbn)
    is("character not allowed", err)

    -- These extra characters are always allowed, even in strict mode.
    local GOOD_WITH_SUPERFLUOUS_CHARS = " 978-0596 527- - 242\t\n\r"
    local isbn, err = Isbn:new(GOOD_WITH_SUPERFLUOUS_CHARS)
    is(GOOD_ISBN, isbn:isbn())
    isbn, err = Isbn:new(GOOD_WITH_SUPERFLUOUS_CHARS, "strict")
    is(GOOD_ISBN, isbn:isbn())
end

function testcase:test_good_isbns ()
    local file = assert(io.open("test/data/isbn13s.txt", "rb"))

    for line in file:lines() do
        local isbn, err = Isbn:new(line)
        assert_table(isbn)
        assert_nil(err)
    end

    file:close()
end

function testcase:test_bad_isbns ()
    local file = assert(io.open("test/data/bad-isbn13s.txt", "rb"))

    for line in file:lines() do
        local isbn, err = Isbn:new(line)
        assert_nil(isbn, "value returned for bad ISBN-13: " .. line)
        assert_string(err, "no string for error message for ISBN-13: " .. line)
    end

    file:close()
end

lunit.run()
-- vi:ts=4 sw=4 expandtab
